/*
 * This file is part of Sponge, licensed under the MIT License (MIT).
 *
 * Copyright (c) SpongePowered <https://www.spongepowered.org>
 * Copyright (c) contributors
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package org.spongepowered.server.util;

import com.flowpowered.math.GenericMath;
import net.minecraft.entity.Entity;
import net.minecraft.util.MovingObjectPosition;
import net.minecraft.util.Vec3;

public final class EntityUtils {

    private EntityUtils() {
    }

    public static MovingObjectPosition rayTraceFromEntity(Entity entity, double traceDistance, float partialTicks) {
        final Vec3 vecPositionEyes = EntityUtils.getPositionEyes(entity, partialTicks);
        final Vec3 vecFacing = entity.func_70676_i(partialTicks);
        final Vec3 vecInFront = vecPositionEyes.func_72441_c(vecFacing.field_72450_a * traceDistance, vecFacing.field_72448_b * traceDistance,
                vecFacing.field_72449_c * traceDistance);
        return entity.field_70170_p.func_147447_a(vecPositionEyes, vecInFront, false, false, true);
    }

    public static Vec3 getPositionEyes(Entity entity, float partialTicks) {
        if (partialTicks == 1.0F) {
            return new Vec3(entity.field_70165_t, entity.field_70163_u + (double) entity.func_70047_e(), entity.field_70161_v);
        }

        double interpX = GenericMath.lerp(entity.field_70169_q, entity.field_70165_t, (double) partialTicks);
        double interpY = GenericMath.lerp(entity.field_70167_r, entity.field_70163_u, (double) partialTicks) + (double) entity.func_70047_e();
        double interpZ = GenericMath.lerp(entity.field_70166_s, entity.field_70161_v, (double) partialTicks);
        return new Vec3(interpX, interpY, interpZ);
    }
}
